#!/usr/bin/env python

'''

  Filename:       qc_list_objects.py
  Description:    qc_list_objects is able to list queues and exchanges available
                  on running Qpid broker (qpidd)
  Author:         Frantisek Reznicek <freznice@redhat.com>
  Usage:          qc_list_objects.py
                    -b|--broker <broker-host>               qpid broker hostname
                    -p|--port <broker-port>                     qpid broker port

                    -e|--list-exchanges                      list exchange names
                    -q|--list-queues                            list queue names
                    -a|--list-all                 list queues and then exchanges

                    -v|--list-properties          list queue/exchange properties

                    --queue-filter <qfilter>      list queues matching <qfilter>
                    --exchange-filter <efilter>      list ex. matching <efilter>

                  Connects to running Qpid broker on host <broker-host> to port
                  <broker-port> and tries to get queue and exchange names
                  -q and -e including queue/exchange properties
                                                    (if --list-properties given)
                  This client uses qpid python qmf interface/API.

  Usage Examples: # get queues from def. broker
                  qc_list_objects.py -b localhost -p 5672 -q

                  # get all queues and ex. incl. props from broker myhost:15688
                  qc_list_objects.py -b myhost -p 15688 -a --list-properties


  Copyright (c) 2008, 2009 Red Hat, Inc. All rights reserved. This copyrighted
  material is made available to anyone wishing to use, modify, copy, or
  redistribute it subject to the terms and conditions of the GNU General
  Public License v.2.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANYWARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

'''

# Import common and qpid modules
# ---------------------------------------------------------------------------
import sys
import os
import re
import optparse
import traceback
import qpid
from qmf.console import Session

import pickle

# constants
# ---------------------------------------------------------------------------
SESSION_TIMEOUT = 20;

FILECOUNT = "qpid.file_count"
FILESIZE  = "qpid.file_size"
MAX_QUEUE_SIZE  = "qpid.max_size"
MAX_QUEUE_COUNT  = "qpid.max_count"
POLICY_TYPE  = "qpid.policy_type"
CLUSTER_DURABLE = "qpid.persist_last_node"
LVQ = "qpid.last_value_queue"
LVQNB = "qpid.last_value_queue_no_browse"
MSG_SEQUENCE = "qpid.msg_sequence"
IVE = "qpid.ive"
QUEUE_EVENT_GENERATION = "qpid.queue_event_generation"
QUEUE_NOLOCAL = "no-local"

# any_to_str() helps to convert any type to string
def any_to_str(in_any):
  int_str='';
  if in_any==None:
    int_str='<default>';
  else:
    try:
      int_str=str(in_any);
    except Exception, e:
      pass;

  return(int_str);

# main function
# ---------------------------------------------------------------------------
def main():

  # parameters definition and parsing
  # -------------------------------------------------------------------------
  usage_msg = "usage: %prog -b <broker-host> -p <broker-port> <-q|-e|-a> ...";
  op = optparse.OptionParser(usage=usage_msg);

  # define parameters
  # -------------------------------------------------------------------------
  op.add_option("-b", "--broker", dest="broker_host", type="string",
                action="store", default=None,
                help="Qpid broker host", metavar="HOST");
  op.add_option("-p", "--port", dest="broker_port", type="int",
                default=None, help="Qpid broker port", metavar="PORT");
  op.add_option("-u", "--user", dest="auth_user", type="string",
                action="store", default=None, metavar="USERID",
                help="Broker authentication access as user");
  op.add_option("--password", dest="auth_pass", type="string",
                action="store", default=None, metavar="PASSID",
                help="Broker authentication user password");

  op.add_option("-e", "--list-exchanges", dest="list_exchanges",
                action="store_true", default=False,
                help="List exchanges");
  op.add_option("-q", "--list-queues", dest="list_queues",
                action="store_true", default=False,
                help="List queues");
  op.add_option("-a", "--list-all", dest="list_all",
                action="store_true", default=False,
                help="List queues and exchages");

  op.add_option("-v", "--list-properties", dest="list_properties",
                action="store_true", default=False,
                help="List queues's and exchages's properties");

  op.add_option("--queue-filter", dest="queue_filter", type="string",
                action="store", default="", metavar="QFILTER",
                help="Queue filter (regexp)");
  op.add_option("--exchange-filter", dest="exchange_filter", type="string",
                action="store", default="", metavar="QFILTER",
                help="Exchange filter (regexp)");


  # parse parameters
  # -------------------------------------------------------------------------
  (options, args) = op.parse_args();

  # detect whether parameters are valid (ex. name, # ex., ex. type)
  # -------------------------------------------------------------------------
  if ( options.list_exchanges == False and options.list_queues == False and \
       options.list_all == False ):
    op.print_help();
    sys.exit(1);

  #  Create a session
  # -------------------------------------------------------------------------
  try:
    # create qmf session
    qmf_session = Session();
    # initiate broker to that session (different versions)
    # TO BE CORRECTED see commented version
    broker = None;
    if(options.broker_host == None):
      if(options.broker_port == None):
        if( (options.auth_user == None) and (options.auth_pass == None) ):
          # no host, no port, no user/pass specified
          broker = qmf_session.addBroker(timeout=SESSION_TIMEOUT);
        else:
          # no host, no port, but user/pass specified
          options.broker_host='localhost';
          broker = qmf_session.addBroker(target='amqp://%s/%s@%s' % \
                    (options.auth_user, options.auth_pass, options.broker_host),
                                         timeout=SESSION_TIMEOUT);
      else:
        if( (options.auth_user == None) and (options.auth_pass == None) ):
          # no host, no user/pass specified, but port
          options.broker_host='localhost';
          broker = qmf_session.addBroker(target='amqp://%s:%d' % \
                                     (options.broker_host, options.broker_port),
                                         timeout=SESSION_TIMEOUT);
        else:
          # no host, but port and user/pass specified
          options.broker_host='localhost';
          broker = qmf_session.addBroker(target='amqp://%s/%s@%s:%d' % \
                                     (options.auth_user, options.auth_pass, \
                                      options.broker_host, options.broker_port),
                                         timeout=SESSION_TIMEOUT);
    else:
      if(options.broker_port == None):
        if( (options.auth_user == None) and (options.auth_pass == None) ):
          # no port, no user/pass, but host specified
          broker = qmf_session.addBroker(target='amqp://%s' % \
                                                          (options.broker_host),
                                         timeout=SESSION_TIMEOUT);
        else:
          # no port, but host and user/pass specified
          broker = qmf_session.addBroker(target='amqp://%s/%s@%s' % \
                                        (options.auth_user, options.auth_pass, \
                                                           options.broker_host),
                                         timeout=SESSION_TIMEOUT);
          
      else:
        if( (options.auth_user == None) and (options.auth_pass == None) ):
          # no user/pass, but host and port specified
          broker = qmf_session.addBroker(target='amqp://%s:%d' % \
                                     (options.broker_host, options.broker_port),
                                         timeout=SESSION_TIMEOUT);
        else:
          # host, port and user/pass specified (all specified)
          broker = qmf_session.addBroker(target='amqp://%s/%s@%s:%d' % \
                                     (options.auth_user, options.auth_pass, \
                                      options.broker_host, options.broker_port),
                                         timeout=SESSION_TIMEOUT);

    agent = None;
    # remember broker agent
    for i_agent in qmf_session.getAgents():
      if i_agent.getAgentBank() == '0':
        agent = i_agent;

  except Exception, e:
    print >> sys.stderr, "Cannot connect to the broker %s:%s" % \
            (any_to_str(options.broker_host), any_to_str(options.broker_port));
    traceback.print_exc();
    sys.exit(1);

  if(agent == None):
    print >> sys.stderr, "Cannot get broker's agent!";
    sys.exit(2);

  # list queues
  # -------------------------------------------------------------------------
  try:
    if (options.list_queues or options.list_all):
      # queue listing
      if options.list_all:
        print "Queues:"

      # get queues
      queues = qmf_session.getObjects(_class="queue", _agent=agent);

      # browse the queues
      for q in queues:
        if re.search (options.queue_filter, q.name):
          print "%s " % (q.name),
          if options.list_properties:
            # get queue properties list
            args = q.arguments
            if q.durable:
              print "durable",
            if CLUSTER_DURABLE in args and args[CLUSTER_DURABLE] == 1:
              print "cluster-durable",
            if q.autoDelete:
              print "auto-del",
            if q.exclusive:
              print "exclusive",
            if FILESIZE in args:
              print "file-size:%d" % args[FILESIZE],
            if FILECOUNT in args:
              print "file-count:%d" % args[FILECOUNT],
            if MAX_QUEUE_SIZE in args:
              print "max-byte-size:%d" % args[MAX_QUEUE_SIZE],
            if MAX_QUEUE_COUNT in args:
              print "max-msg-count:%d" % args[MAX_QUEUE_COUNT],
            if POLICY_TYPE in args:
              print "limit-policy:%s" % args[POLICY_TYPE].replace("_", "-"),
            if LVQ in args and args[LVQ] == 1:
              print "order:lvq",
            if LVQNB in args and args[LVQNB] == 1:
              print "order:lvq-no-browse",
            if QUEUE_EVENT_GENERATION in args:
              print "generate-queue-events:%d" % args[QUEUE_EVENT_GENERATION],
            if QUEUE_NOLOCAL in args:
              print "no-local",

            print ( "msg|byteEnq/Deq:%d/%d|%d/%d unacked:%d " + \
                    "consumer_cnt:%d binding_cnt:%d" ) % \
                  (q.msgTotalEnqueues, q.msgTotalDequeues,
                   q.byteTotalEnqueues, q.byteTotalDequeues,
                   q.unackedMessages, q.consumerCount, q.bindingCount),
          print


    # list queues
    # -------------------------------------------------------------------------
    if (options.list_exchanges or options.list_all):
      # queue listing
      if options.list_all:
        print "Exchanges:"

      # get queues
      exchanges = qmf_session.getObjects(_class="exchange", _agent=agent);

      # browse the queues
      for ex in exchanges:
        if re.search (options.exchange_filter, ex.name):
          print "%s " % ex.name,
          if options.list_properties:
            print "type.%s " % ex.type,
            # get exchange properties list
            args = ex.arguments
            if ex.durable:
              print "durable",
            if MSG_SEQUENCE in args and args[MSG_SEQUENCE] == 1:
              print "sequence",
            if IVE in args and args[IVE] == 1:
              print "ive",

            print ( "msg|byteRx/Drop/Rt:%d/%d/%d|%d/%d/%d " + \
                    "producer_cnt:%d binding_cnt:%d" ) % \
                  (ex.msgReceives, ex.msgDrops, ex.msgRoutes,
                   ex.byteReceives, ex.byteDrops, ex.byteRoutes,
                   ex.producerCount, ex.bindingCount),

          print

  except Exception, e:
    print >> sys.stderr, \
          "Cannot get queue / exchange names, an qmf exception thrown";
    traceback.print_exc();
    #qmf_session.close(timeout=SESSION_TIMEOUT);
    sys.exit(1);

  # clean-up actions ...(close session (QPID-2730 in devel) + delete brokers)
  # -------------------------------------------------------------------------
  if (qmf_session != None) and (broker != None):
    qmf_session.delBroker(broker);
  #qmf_session.close(timeout=SESSION_TIMEOUT);

# main() call
# ---------------------------------------------------------------------------
if __name__ == "__main__":
    main()


# ---------------------------------------------------------------------------
# eof
# ---------------------------------------------------------------------------
